// Copyright (c)2002 Eric Carlson
// This source is licensed under the BSD license.
// Please see the LICENSE.txt file included with the distribution for more information.

#import "USBMonitor.h"
#import "USBMonitorControlClassSpec.h"

@implementation USBMonitor

- (id)init
{
    BOOL found = FALSE;
    pRecDevice theDevice;
    pRecElement anElement;
    
    self = [super init];

    //get the device list so we can iterate through it
    HIDBuildDeviceList(0,0);

    //iterate through device list looking for the 900U monitor
    theDevice = HIDGetFirstDevice();
    while(theDevice)
    {
        if( (theDevice->productID == kMitsuDiamondPro900U_ProdID) &&
            (theDevice->vendorID == kMitsuDiamondPro900U_VendID) )
        {
            //we found the monitor, so get references to all its elements
            anElement = HIDGetFirstDeviceElement(theDevice,kHIDElementTypeAll);
            while(anElement)
            {
                if(anElement->usage > MYELEMENTS_LENGTH-1)
                {
                    NSLog(@"Element usage value out of range: %d > %d",anElement->usage, sizeof(myElements)-1);
                    return nil;
                }

                if(anElement->usagePage == kHIDPage_VESA)
                {
                    myElements[anElement->usage] = anElement; //put it in our array of elements              
                }
                
                anElement = HIDGetNextDeviceElement(anElement,kHIDElementTypeAll);
            }
            found = TRUE;
            break; //found the device so exit loop (for now we only care about 1 device)
        } //end check if this is the device we want

        theDevice = HIDGetNextDevice(theDevice);
    } //end device search

    if(found)
    {
        myMonitor = theDevice; //set the global pointer to the monitor we found
        return self; //done
    } else {
        myMonitor = NULL;
        return nil;
    }

}

- (void)dealloc
{
    NSLog(@"USBMonitor: dealloc called\n");
    HIDReleaseDeviceList();
    [super dealloc]; //last
}

#pragma mark ----------------set values----------------
- (void)updateSettingsUsingArray:(long*)settings
            withProgressSelector:(SEL)updateProgress
                selectorWhenDone:(SEL)finish
                         ofClass:(NSObject *)callbackObject
{
    int i;
    for(i=0; i<MYELEMENTS_LENGTH; i++)
    {
        if(settings[i]!=LONG_MIN) {
            [self setHIDElement:i withValue:(float)settings[i]];
        }
        [callbackObject performSelector:updateProgress];
    }
    [callbackObject performSelector:finish];
}

- (BOOL)degauss
{
    return [self setHIDElement:kHIDVESAUsagePage_Degauss withValue:1];
}

- (BOOL)setHorizontalPosition:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizPosition withValue:value];
}

- (BOOL)setHorizontalSize:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizSize withValue:value];
}

- (BOOL)setVerticalPosition:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_VertPosition withValue:value];
}

- (BOOL)setVerticalSize:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_VertSize withValue:value];
}

- (BOOL)setBrightness:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_Brightness withValue:value];
}

- (BOOL)setContrast:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_Contrast withValue:value];
}

- (BOOL)setHorizontalPincushion:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizPin withValue:value];
}

- (BOOL)setHorizontalPincushionBalance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizPinBal withValue:value];
}

- (BOOL)setHorizontalMisconvergence:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_HorizMisConv withValue:value];
}

- (BOOL)setVerticalMisconvergence:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_VertMisConv withValue:value];
}

- (BOOL)setKeystoneBalance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_KeyBalance withValue:value];
}

- (BOOL)setKeystone:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_Key withValue:value];
}

- (BOOL)setRotation:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_Tilt withValue:value];
}

- (BOOL)setTopCornerDistortionControl:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_TCDistControl withValue:value];
}

- (BOOL)setTopCornerDistortionBalance:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_TCDistBalance withValue:value];
}

- (BOOL)setBottomCornerDistortionControl:(float)value
{
    return [self setHIDElement:kHIDVESAUsagePage_BCDistControl withValue:value];
}


#pragma mark ----------------get values----------------
//caller is responsible for freeing result
- (long*)settingsArray
{
    long *settings;
    int i;

    settings = (long*) malloc(sizeof(long)*MYELEMENTS_LENGTH);
    for(i=0; i<MYELEMENTS_LENGTH; i++)
    {
        if( (myElements[i] != NULL) && (i != kHIDVESAUsagePage_Degauss) ) //skip degauss usage page!
        {
            settings[i] = [self getHIDElementValue:myElements[i]->usage];
        } else {
            settings[i] = LONG_MIN;
        }
    }
    return settings;
}

- (float)brightness
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_Brightness];
}

- (float)contrast
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_Contrast];
}

- (float)horizontalPosition
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizPosition];
}

- (float)horizontalSize
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizSize];
}

- (float)verticalPosition
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_VertPosition];
}

- (float)verticalSize
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_VertSize];
}

- (float)horizontalPincushion
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizPin];
}

- (float)horizontalPincushionBalance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizPinBal];
}

- (float)horizontalMisconvergence
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_HorizMisConv];
}

- (float)verticalMisconvergence
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_VertMisConv];
}

- (float)keystoneBalance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_KeyBalance];
}

- (float)keystone
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_Key];
}

- (float)rotation
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_Tilt];
}

- (float)topCornerDistortionControl
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_TCDistControl];
}

- (float)topCornerDistortionBalance
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_TCDistBalance];
}

- (float)bottomCornerDistortionControl
{
    return (float) [self getHIDElementValue:kHIDVESAUsagePage_BCDistControl];
}


#pragma mark ----------------private methods----------------
- (BOOL)setHIDElement:(int)key withValue:(long)value
{
    pRecElement thisElement;
    IOHIDEventStruct myEvent;

    thisElement = myElements[key];

    HIDTransactionGetElementValue(myMonitor,thisElement,&myEvent);
    myEvent.value = value;
    HIDSetElementValue(myMonitor,thisElement,&myEvent);

    return TRUE;
}


- (long)getHIDElementValue:(int)key
{
    pRecElement thisElement;
    long value;
    
    thisElement = myElements[key];
    value = HIDGetElementValue(myMonitor,thisElement);
    return value;    
}

@end
